//-------------------------------------------------------------------
//  PROGRAM:          PYRAMID.CPP
//
//  DESCRIPTION:      Sample program that utilizes Borland's
//                    OWL
//
//                    Written by Carlos Yu
//
//  NOTE:             The program design is kinda hacked because
//                    this is my first attempt at creating a
//                    fully functional Windows program using
//                    OWL and C++.
//
//                    Any suggestions on making the program better,
//                    specially the DealCards module which is
//                    painfully slow, would be greatly appreciated.
//
//                    Compuserve ID 72672,1567                    
//-------------------------------------------------------------------


//--------------------------------
//  Include files
//--------------------------------

#include <owl.h>                              
#include <bwcc.h>                             
#include <stdio.h>                              
#include <string.h>
#include <dialog.h>                             
#include <button.h>                            
#include <static.h>                           
#include <bbutton.h>                            
#include <bgrpbox.h>                           
#include <tcollect.h>                        



//--------------------------------
//  User Include Files
//--------------------------------

#include "cardwin.h"
#include "CDialog.h"

//-------------------------------------------------------------------
//  GLOBAL VARIABLES AND DEFINITIONS
//-------------------------------------------------------------------
//--------------------------------
//  Menu Item Constants
//--------------------------------

#define       CM_DEAL          101
#define       CM_OPTIONS       102
#define       CM_QUIT          103
#define       CM_INSTRUCT      104
#define       CM_ABOUT         105


//--------------------------------
//  Push Button Constants
//--------------------------------

#define       ID_DEAL          111
#define       ID_OPTIONS       112
#define       ID_QUIT          113
#define       ID_INSTRUCT      114
#define       ID_ABOUT         115

//--------------------------------
//  Custom Control Constants
//--------------------------------

#define       ID_STATUSGROUP   121
#define       ID_STATUSTEXT    122


//--------------------------------
//  Constant variables
//--------------------------------

#define       DIALOGQUIT       400
#define       DIALOGPLAY       800
#define       DIALOGABOUT      500
#define       DIALOGINSTRUCT   600

#define       MAINMENU         100
#define       MAINACCEL        100
#define       MAINICON         100

#define       WINWIDTH         301
#define       WINHEIGHT        240

#define       STATUSLEN        255

#define       WINBITMSG        589
#define       BACKGROUNDBIT   7777

#define       ORIGIN_X          43
#define       ORIGIN_Y          20

#define       CHILD_CLICK       20


POINT PYRAMID_POS[] =
{
    { ORIGIN_X + 222, ORIGIN_Y },
    { ORIGIN_X + 185, ORIGIN_Y +  25 },
    { ORIGIN_X + 259, ORIGIN_Y +  25 },
    { ORIGIN_X + 148, ORIGIN_Y +  50 },
    { ORIGIN_X + 222, ORIGIN_Y +  50 },
    { ORIGIN_X + 296, ORIGIN_Y +  50 },
    { ORIGIN_X + 111, ORIGIN_Y +  75 },
    { ORIGIN_X + 185, ORIGIN_Y +  75 },
    { ORIGIN_X + 259, ORIGIN_Y +  75 },
    { ORIGIN_X + 333, ORIGIN_Y +  75 },
    { ORIGIN_X +  74, ORIGIN_Y + 100 },
    { ORIGIN_X + 148, ORIGIN_Y + 100 },
    { ORIGIN_X + 222, ORIGIN_Y + 100 },
    { ORIGIN_X + 296, ORIGIN_Y + 100 },
    { ORIGIN_X + 370, ORIGIN_Y + 100 },
    { ORIGIN_X +  37, ORIGIN_Y + 125 },
    { ORIGIN_X + 111, ORIGIN_Y + 125 },
    { ORIGIN_X + 185, ORIGIN_Y + 125 },
    { ORIGIN_X + 259, ORIGIN_Y + 125 },
    { ORIGIN_X + 333, ORIGIN_Y + 125 },
    { ORIGIN_X + 407, ORIGIN_Y + 125 },
    { ORIGIN_X,       ORIGIN_Y + 150 },
    { ORIGIN_X +  74, ORIGIN_Y + 150 },
    { ORIGIN_X + 148, ORIGIN_Y + 150 },
    { ORIGIN_X + 222, ORIGIN_Y + 150 },
    { ORIGIN_X + 296, ORIGIN_Y + 150 },
    { ORIGIN_X + 370, ORIGIN_Y + 150 },
    { ORIGIN_X + 444, ORIGIN_Y + 150 }
};


/*******************************************************************
 *******************************************************************
 ************************  PROGRAM STARTS  *************************
 *******************************************************************
 *******************************************************************/
//-------------------------------------------------------------------
//  CLASS:            TPyramidApp
//
//  DESCRIPTION:      Derive TApplication to create TPyramidApp 
//
//  RETURNS:          NONE
//-------------------------------------------------------------------         

class TPyramidApp : public TApplication
{
public:
    TPyramidApp(LPSTR AName, HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpCmdLine, int nCmdShow) : TApplication(AName, hInstance,
        hPrevInstance, lpCmdLine, nCmdShow) {};

    virtual void InitInstance();
    virtual void InitMainWindow();
};


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  DESCRIPTION:      Derive TWindow to create TPyramidWin which is
//                    the main window class of the application
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

_CLASSDEF(TPyramidWin)
class TPyramidWin : public TWindow
{
private:
    int               tagIdx,
                      playIdx,
                      childIndex,
                      discardCount;

    char              statusMsg[255];

    BOOL              isTagged,
                      hasDealt,
                      playOccupied;

    HMENU             MenuHandle,
                      MenuPopup1,
                      MenuPopup2,
                      MenuSystem;

    HBRUSH            GrayBrush;
    HBITMAP           hBitBack;

    PTStatic          StatusText;
    PTButton          PushDeal,
                      PushAbout,
                      PushInstruct,
                      PushQuit;
    PTBGroupBox       StatusGroup;
    PTNSCollection    allObjects;
    PTCardWin         cardPtr[53];

public:

    // Constructor

    TPyramidWin(PTWindowsObject AParent, LPSTR ATitle);

    //----------------------------
    //  Window default functions
    //         and events
    //----------------------------

    virtual BOOL  CanClose();
    virtual void  SetupWindow();
    virtual LPSTR GetClassName();
    virtual void  GetWindowClass(WNDCLASS& AWndClass);

    virtual void  WMMove(RTMessage Msg)
        = [WM_FIRST + WM_MOVE];

    virtual void  WMEraseBkgnd(RTMessage)
        = [WM_FIRST + WM_ERASEBKGND];

    virtual void  WMDestroy(RTMessage Msg)
        = [WM_FIRST + WM_DESTROY];

    virtual void  WMCtlColor(RTMessage Msg)
        = [WM_FIRST + WM_CTLCOLOR];

    virtual void  WMMouseMove(RTMessage)
        = [WM_FIRST + WM_MOUSEMOVE];

    virtual void  WMMenuSelect(RTMessage Msg)
        = [WM_FIRST + WM_MENUSELECT];

    virtual void  WMLButtonDown(RTMessage Msg)
        = [WM_FIRST + WM_LBUTTONDOWN];

    virtual void  WMRButtonDown(RTMessage Msg)
        = [WM_FIRST + WM_RBUTTONDOWN];

    //----------------------------
    //  User functions
    //----------------------------

    virtual void  WMNotifyClick(RTMessage Msg)
        = [WM_USER + CHILD_CLICK];

    void Animate();

    BOOL TestKing();

    void DealCards();

    void PlayAgain();

    BOOL ProcessTag();

    void ReturnToDeck();

    void InitializeDeck();

    int  GetValue(int Value);


    //---------------
    //  Push Button
    //---------------

    virtual void  IDQuit(RTMessage)
        = [ID_FIRST + ID_QUIT];

    virtual void  IDDeal(RTMessage)
        = [ID_FIRST + ID_DEAL];

    virtual void  IDInstruct(RTMessage)
        = [ID_FIRST + ID_INSTRUCT];

    virtual void  IDAbout(RTMessage)
        = [ID_FIRST + ID_ABOUT];

    //---------------
    //  Menu Items
    //---------------

    virtual void  CMDeal(RTMessage)
        = [CM_FIRST + CM_DEAL];

/* virtual void  CMOptions(RTMessage)
        = [CM_FIRST + CM_OPTIONS];    */

    virtual void  CMQuit(RTMessage)
        = [CM_FIRST + CM_QUIT];

    virtual void  CMInstruct(RTMessage)
        = [CM_FIRST + CM_INSTRUCT];    

    virtual void  CMAbout(RTMessage)
        = [CM_FIRST + CM_ABOUT]; 
};


//-------------------------------------------------------------------
//  CLASS:            TPyramidApp
//
//  MEMBER:           InitInstance
//
//  DESCRIPTION:      Initialize the application's instance.
//
//                    In this particular case, the application may
//                    have multiple instances.  However, the code
//                    that prevents multiple instances will be
//                    included as a comment.
//
//                    The accelerator resource is also loaded in
//                    this function.
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidApp::InitInstance()
{
//--------------------------------
//  Use this for a single
//  instance of the application
//--------------------------------

//  if(hPrevInstance == 0)
//  {
//      TApplication::InitInstance();
//  }
//  else
//  {
//      MessageBox(0, "This program is already running",
//          "Application Error", MB_TASKMODAL | MB_ICONEXCLAMATION);
//      exit(1);
//  }

//--------------------------------
//  Use this to allow multiple
//  instances of the application
//--------------------------------

    TApplication::InitInstance();
    HAccTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(MAINACCEL));
}           


//-------------------------------------------------------------------
//  CLASS:            TPyramidApp
//
//  MEMBER:           InitMainWindow
//
//  DESCRIPTION:      Initialize the application's main window
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidApp::InitMainWindow()
{
    MainWindow = new TPyramidWin(NULL, "CYSoft Pyramid");
}

//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           Constructor
//
//  DESCRIPTION:      Class constructor for the main window.  Make
//                    the window size a little smaller than the
//                    screen size as long as the minimum or maximum
//                    sizes are not exceeded in WM_GETMINMAXINFO.
//
//  ARGUMENTS:        AParent - pointer to a parent window
//                    ATitle  - window caption title
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

TPyramidWin::TPyramidWin(PTWindowsObject AParent, LPSTR ATitle) :
    TWindow(AParent, ATitle)
{
    AssignMenu(MAINMENU);

    int screenWidth  = GetSystemMetrics(SM_CXSCREEN) / 2;
    int screenHeight = GetSystemMetrics(SM_CYSCREEN) / 2; 

    Attr.X     = screenWidth  - WINWIDTH;
    Attr.Y     = screenHeight - WINHEIGHT; 
    Attr.H     = WINHEIGHT * 2;
    Attr.W     = WINWIDTH  * 2;
    Attr.Style = WS_CAPTION |
                 WS_SYSMENU |
                 WS_MINIMIZEBOX |
                 WS_CLIPCHILDREN;

    EnableKBHandler();
}                    


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           SetupWindow
//
//  DESCRIPTION:      Create all child window/controls and store
//                    the pointers in a collection
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::SetupWindow()
{
    TWindow::SetupWindow();

    //----------------------------
    //  Initialize variables
    //----------------------------

    tagIdx = 0;
    isTagged = FALSE;
    hasDealt = FALSE;
    playOccupied = FALSE;

    InitializeDeck();

    //----------------------------
    //  Get the popup handles
    //----------------------------

    MenuHandle = GetMenu(HWindow);
    MenuPopup1 = GetSubMenu(MenuHandle, 0);
    MenuPopup2 = GetSubMenu(MenuHandle, 1);
    MenuSystem = GetSystemMenu(HWindow, FALSE);

    hBitBack = LoadBitmap(GetApplication()->hInstance,
        MAKEINTRESOURCE(BACKGROUNDBIT));

    GrayBrush = CreateSolidBrush(RGB(192, 192, 192));

    //----------------------------
    //  Initialize children
    //----------------------------

    StatusGroup  = new TBGroupBox(this,   ID_STATUSGROUP,  NULL, 24,
        281, 552, 24);

    StatusText   = new TStatic(this, ID_STATUSTEXT, "Ready", 30,
        284, 543, 20, STATUSLEN);

    StatusText->Attr.Style = (StatusText->Attr.Style & ~SS_LEFT) | SS_CENTER;


    PushDeal = new TButton(this, ID_DEAL, "&Deal", 358, 312, 100, 22, FALSE);

    PushAbout = new TButton(this, ID_ABOUT, "&About...", 358, 344,
         100, 22, FALSE);

    PushInstruct = new TButton(this, ID_INSTRUCT, "&Instructions", 358, 376,
         100, 22, FALSE);
          
    PushQuit = new TButton(this, ID_QUIT, "&Quit", 358, 408, 100, 22, FALSE);

    //----------------------------
    //  Create the child windows
    //----------------------------

    GetApplication()->MakeWindow(StatusGroup);
    GetApplication()->MakeWindow(StatusText);
    GetApplication()->MakeWindow(PushDeal);
    GetApplication()->MakeWindow(PushAbout);
    GetApplication()->MakeWindow(PushInstruct);
    GetApplication()->MakeWindow(PushQuit);

    //----------------------------
    //  Store in collection
    //----------------------------

    allObjects = new TNSCollection(100, 20);

    allObjects->insert(StatusGroup);
    allObjects->insert(StatusText);
    allObjects->insert(PushDeal);
    allObjects->insert(PushAbout);
    allObjects->insert(PushInstruct);
    allObjects->insert(PushQuit);
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           GetWindowClass
//
//  DESCRIPTION:      Fill the WNDCLASS structure with the relevant
//                    arguments such as the icon and background.
//
//  ARGUMENTS:        AWndClass - WNDCLASS strucrure of windows API
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::GetWindowClass(WNDCLASS& AWndClass)
{
    TWindow::GetWindowClass(AWndClass);

    AWndClass.hbrBackground = NULL;
    AWndClass.hIcon = LoadIcon(GetApplication()->hInstance,
        MAKEINTRESOURCE(MAINICON));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           GetClassName
//
//  DESCRIPTION:      Return the window class name
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          LPSTR which holds the window class name
//-------------------------------------------------------------------

LPSTR TPyramidWin::GetClassName()
{
    return "TPyramidWin";
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMDestroy
//
//  DESCRIPTION:      Clean up before exit
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMDestroy(RTMessage Msg)
{
    if ( hasDealt )
        for ( int loop = 1; loop < 53; loop++ )
            delete(cardPtr[loop]);

    DeleteObject(GrayBrush);
    DeleteObject(hBitBack);

    allObjects->freeAll();
    delete(allObjects);

    TWindow::WMDestroy(Msg);
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMMove
//
//  DESCRIPTION:      Update Attr.X and Y when window moves
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMMove(RTMessage Msg)
{
    Attr.X = (int) Msg.LP.Lo;
    Attr.Y = (int) Msg.LP.Hi;
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMEraseBkgnd
//
//  DESCRIPTION:      Fill the background with the bitmap background
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMEraseBkgnd(RTMessage)
{
    HDC     hDC;

    hDC = GetDC(HWindow);
    HDC hMemDC = CreateCompatibleDC(hDC);
    HDC TempDC = SelectObject(hMemDC, hBitBack);
    BitBlt(hDC, 0, 0, 600, 440, hMemDC, 0, 0, SRCCOPY);

    SelectObject(hMemDC, TempDC);
    DeleteDC(hMemDC);
    ReleaseDC(HWindow, hDC);
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMCtlColor
//
//  DESCRIPTION:      Change the color of custom controls
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMCtlColor(RTMessage Msg)
{
    HDC  &hDC    = (HDC)  Msg.WParam;
    HWND hwnd    = (HWND) Msg.LP.Lo;
    int  CtlType = (int)  Msg.LP.Hi;

    if(CtlType == CTLCOLOR_STATIC)
    {
        if(hwnd == StatusText->HWindow)
        {
            SetTextColor(hDC, RGB(  0,   0,   0));
            SetBkColor  (hDC, RGB(192, 192, 192));

            Msg.Result = (DWORD) GrayBrush;
        }
        else
            DefWndProc(Msg);
    }
    else
        DefWndProc(Msg);
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMMenuSelect
//
//  DESCRIPTION:      Send appropriate string to status bar when
//                    a menu item is highlighted
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMMenuSelect(RTMessage Msg)
{
    if(Msg.LP.Hi == NULL && Msg.LP.Lo == 0xffff)
    {
        // Exiting menu mode.

        strcpy(statusMsg, "Ready");
        StatusText->SetText(statusMsg);
        return;
    }
    else if (Msg.LP.Lo & MF_SYSMENU)
    {
        //
        // System menu is up
        //
        if ((HMENU)Msg.LP.Hi == MenuSystem)
        {
            //
            // System menu item is selected
            //
            // System menu: item contains SC_* code.

            switch((int)Msg.WParam)
            {
                case SC_SIZE:
                    strcpy(statusMsg, "Size the window with the keyboard");
                    break;
                case SC_MOVE:
                    strcpy(statusMsg, "Move the window with the keyboard");
                    break;
                case SC_MINIMIZE:
                    strcpy(statusMsg, "Minimize the window");
                    break;
                case SC_MAXIMIZE:
                    strcpy(statusMsg, "Maximize the window");
                    break;
                case SC_CLOSE:
                    strcpy(statusMsg, "Close the window");
                    break;
                case SC_RESTORE:
                    strcpy(statusMsg, "Restore the window");
                    break;
                case SC_TASKLIST:
                    strcpy(statusMsg, "Switch to another window");
                    break;
            }
        }
        else if(Msg.LP.Lo & MF_POPUP)
            strcpy(statusMsg, "System Menu (Main Window Manipulation)");
    }
    else if(Msg.LP.Lo & MF_POPUP)
    {
        // Pulling down a popup submenu: Msg.LP.Hi contains popup handle.

        if((HMENU)Msg.WParam == MenuPopup1)
            strcpy(statusMsg, "Game Menu (deal, options, etc...)");
        else if((HMENU)Msg.WParam == MenuPopup2)
            strcpy(statusMsg, "Access Help Items");
    }
    else
    {
        UINT  itemIndex =  (UINT) Msg.WParam;
        itemIndex -= 100;

        LoadString(GetApplication()->hInstance,
            itemIndex, statusMsg, 254);
    }

    StatusText->SetText(statusMsg);
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMMouseMove
//
//  DESCRIPTION:      Make sure "Ready" is displayed in status bar
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMMouseMove(RTMessage)
{
    if((strcmp(statusMsg, "Ready")) != 0)
    {
        strcpy(statusMsg, "Ready");
        StatusText->SetText(statusMsg);
    }
}




//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMRButtonDown
//
//  DESCRIPTION:      Cheat by reshuffling the deck pile
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          NONE
//
//  NOTE:             Ninety percent of the time, the cards in the
//                    pyramid will all be discarded.  The hardest
//                    part is getting the deck in order so that
//                    all cards will be matched properly.  By
//                    hitting the right mouse button in the deck
//                    area, the cards are reshuffled and a win is
//                    assured if there are no cards remaining in
//                    the pyramid.
//-------------------------------------------------------------------

void TPyramidWin::WMRButtonDown(RTMessage Msg)
{
    if ( ((int) Msg.LP.Lo >=  34) &&
         ((int) Msg.LP.Lo <= 104) &&
         ((int) Msg.LP.Hi >= 321) &&
         ((int) Msg.LP.Hi <= 411) )
    {
        MessageBeep(0);

        int count = 0;

        int tempArray[53];

        for ( int loop = 1; loop < 53; loop++ )
        {
            if ( (cardPtr[loop]->where == INDECK)  ||
                 (cardPtr[loop]->where == INWASTE) ||
                 (cardPtr[loop]->where == INPLAY) )
            {
                tempArray[count] = loop;
                cardPtr[loop]->where = INWASTE;
                count++;
            }
        }

        randomize();

        for ( loop = 0; loop < count; loop++ )
        {
            int randIdx = random(count - 1);

            int tempVal = cardPtr[tempArray[loop]]->cardValue;
            int tempBit = cardPtr[tempArray[loop]]->bitNum;

            cardPtr[tempArray[loop]]->cardValue
                = cardPtr[tempArray[randIdx]]->cardValue;

            cardPtr[tempArray[loop]]->bitNum
                = cardPtr[tempArray[randIdx]]->bitNum;

            cardPtr[tempArray[randIdx]]->cardValue = tempVal;
            cardPtr[tempArray[randIdx]]->bitNum = tempBit;
        }
    }

    SendMessage(HWindow, WM_LBUTTONDOWN, NULL, MAKELPARAM(40, 330));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           WMLButtonDown
//
//  DESCRIPTION:      Bring cards back to the deck pile
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::WMLButtonDown(RTMessage Msg)
{
    BOOL bFlag   = TRUE;

    if ( ((int) Msg.LP.Lo >=  34) &&
         ((int) Msg.LP.Lo <= 104) &&
         ((int) Msg.LP.Hi >= 321) &&
         ((int) Msg.LP.Hi <= 411) )
    {
        playOccupied = FALSE;

        for ( int loop = 52; loop > 0; loop-- )
        {
            if ( (cardPtr[loop]->where == INDECK) ||
                 (cardPtr[loop]->where == INPLAY) ||
                 (cardPtr[loop]->where == INWASTE) )
            {
                cardPtr[loop]->where = INDECK;
                cardPtr[loop]->faceUp = FALSE;

                InvalidateRect(cardPtr[loop]->HWindow, NULL, TRUE);

                if ( bFlag )
                {

                    MoveWindow(cardPtr[loop]->HWindow, DECK_X,
                        DECK_Y - 1, CARDWIDTH, CARDHEIGHT, TRUE);  

                    SetWindowPos(cardPtr[loop]->HWindow, HWND_BOTTOM,
                        DECK_X, DECK_Y - 1, CARDWIDTH, CARDHEIGHT,
                        SWP_SHOWWINDOW);

                    bFlag = FALSE;
                }
                else
                {
                    MoveWindow(cardPtr[loop]->HWindow, DECK_X,
                        DECK_Y + 1, CARDWIDTH, CARDHEIGHT, TRUE);  

                    SetWindowPos(cardPtr[loop]->HWindow, HWND_BOTTOM,
                        DECK_X, DECK_Y + 1, CARDWIDTH, CARDHEIGHT,
                        SWP_SHOWWINDOW);

                    bFlag = TRUE;
                }

                InvalidateRect(cardPtr[loop]->HWindow, NULL, TRUE);
            }
        }
    }
}
                 


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           CanClose
//
//  DESCRIPTION:      Verify program exit
//
//  ARGUMENTS:        NONE
//
//  RETURNS:          TRUE or FALSE
//-------------------------------------------------------------------

BOOL TPyramidWin::CanClose()
{
    BOOL bResult = TRUE;
    bResult = GetApplication()->ExecDialog(new CDialog(this, DIALOGQUIT));
    return(bResult);
}

//-------------------------------------------------------------------
//  CLASS:              TPYramidWin
//
//  MEMBER:             DealCards
//
//  DESCRIPTION:        Deal the cards
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            NONE
//-------------------------------------------------------------------

void TPyramidWin::DealCards()
{
    hasDealt = TRUE;
    MessageBeep(0);
    discardCount = 0;
    playOccupied = FALSE;

    StatusText->SetText("Shuffling Cards");

    //----------------------------
    //  Shuffle cards
    //----------------------------

    randomize();

    for ( int replace = 1; replace < 53; replace++)
    {
        int randomCard = random(51);
        randomCard++;
        int tempBit   = cardPtr[replace]->bitNum;
        int tempValue = cardPtr[replace]->cardValue;
        cardPtr[replace]->bitNum       = cardPtr[randomCard]->bitNum;
        cardPtr[replace]->cardValue    = cardPtr[randomCard]->cardValue;
        cardPtr[randomCard]->bitNum    = tempBit;
        cardPtr[randomCard]->cardValue = tempValue;
    }

    //----------------------------
    //  Show the window
    //----------------------------


    for ( int loop = 52; loop > 0; loop-- )
    {
        if ( loop > 28 )
        {
            cardPtr[loop]->faceUp = FALSE;
            cardPtr[loop]->where  = INDECK;
        }
        else if ( loop > 21 )
        {
            cardPtr[loop]->where  = INPYRAMID;
            cardPtr[loop]->faceUp = TRUE;
        }
        else
        {
            cardPtr[loop]->where  = INPYRAMID;
            cardPtr[loop]->faceUp = FALSE;
        }

        cardPtr[loop]->cardTagged = FALSE;

        SetWindowPos(cardPtr[loop]->HWindow, HWND_BOTTOM, cardPtr[loop]->xPos,
            cardPtr[loop]->yPos, CARDWIDTH, CARDHEIGHT, SWP_NOSIZE);
    }

    StatusText->SetText("Ready");
}


//-------------------------------------------------------------------
//  CLASS:              TPYramidWin
//
//  MEMBER:             PlayAgain
//
//  DESCRIPTION:        Ask user to play another
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            NONE
//-------------------------------------------------------------------

void TPyramidWin::PlayAgain()
{
    BOOL bResult = TRUE;

    bResult = GetApplication()->ExecDialog(new CDialog(this, DIALOGPLAY));

    if ( bResult )
        DealCards();
    else
        CloseWindow();
}


//-------------------------------------------------------------------
//  CLASS:              TPYramidWin
//
//  MEMBER:             Initialize Deck
//
//  DESCRIPTION:        Create card deck
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            NONE
//-------------------------------------------------------------------

void TPyramidWin::InitializeDeck()
{
    int x,
        y,
        deckIndex;

    for ( deckIndex = 1; deckIndex < 53; deckIndex++ )
    {
        //------------------------
        //  X nad Y coordinates
        //------------------------

        if ( deckIndex < 29 )
        {
            x = PYRAMID_POS[deckIndex - 1].x;
            y = PYRAMID_POS[deckIndex - 1].y;
        }
        else
        {
            x = DECK_X;
            y = DECK_Y;
        }

        //------------------------
        //  Create Window Pointer
        //------------------------

        cardPtr[deckIndex] = new TCardWin(this, NULL, deckIndex,
            x, y, GetApplication()->hInstance);

        //------------------------
        //  Fill in default values
        //------------------------

        if ( deckIndex < 22 )
        {
            cardPtr[deckIndex]->faceUp = FALSE;
            cardPtr[deckIndex]->bitNum = deckIndex;
            cardPtr[deckIndex]->cardTagged = FALSE;
            cardPtr[deckIndex]->where  = INPYRAMID;
            cardPtr[deckIndex]->cardValue = GetValue(deckIndex);
        }
        else
            if ( deckIndex < 29 )
            {
                cardPtr[deckIndex]->faceUp = TRUE;
                cardPtr[deckIndex]->bitNum = deckIndex;
                cardPtr[deckIndex]->cardTagged = FALSE;
                cardPtr[deckIndex]->cardValue = GetValue(deckIndex);
                cardPtr[deckIndex]->where = INPYRAMID;
            }
            else
                {
                    cardPtr[deckIndex]->faceUp = FALSE;
                    cardPtr[deckIndex]->bitNum = deckIndex;
                    cardPtr[deckIndex]->cardTagged = FALSE;
                    cardPtr[deckIndex]->cardValue = GetValue(deckIndex);
                    cardPtr[deckIndex]->where = INDECK;
                }

        GetApplication()->MakeWindow(cardPtr[deckIndex]);
    }
}


//-------------------------------------------------------------------
//  CLASS:              TPyramidWin
//
//  MEMBER:             TestKing
//
//  DESCRIPTION:        If the card is a king, move it to the
//                      discard pile
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            TRUE if king, FALSE if not
//-------------------------------------------------------------------

BOOL TPyramidWin::TestKing()
{
    if ( cardPtr[childIndex]->cardValue == 13 )
    {
        if ( cardPtr[childIndex]->where == INPLAY )
            playOccupied = FALSE;

        cardPtr[childIndex]->where = INDISCARD;

        SetWindowPos(cardPtr[childIndex]->HWindow, HWND_TOP, DISCARD_X,
            DISCARD_Y, CARDWIDTH, CARDHEIGHT, SWP_NOSIZE);

        InvalidateRect(cardPtr[childIndex]->HWindow, NULL, TRUE);

        discardCount++;
        return TRUE;
    }

    return FALSE;
}


//-------------------------------------------------------------------
//  CLASS:              TPyramid
//
//  MEMBER:             ProcessTag
//
//  DESCRIPTION:        Tag or untag or match cards
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            TRUE if updated, FALSE otherwise
//-------------------------------------------------------------------

BOOL TPyramidWin::ProcessTag()
{
    if ( cardPtr[childIndex]->cardTagged )
    {
        cardPtr[childIndex]->cardTagged = FALSE;
        isTagged = FALSE;
        InvalidateRect(cardPtr[childIndex]->HWindow, NULL, TRUE);
    }
    else
        {
            if ( isTagged )
            {
                if ( cardPtr[childIndex]->cardValue +
                    cardPtr[tagIdx]->cardValue == 13 )
                {
                    if ( (cardPtr[childIndex]->where == INPLAY) ||
                         (cardPtr[tagIdx]->where == INPLAY) )
                            playOccupied = FALSE;

                    cardPtr[childIndex]->where = INDISCARD;
                    cardPtr[childIndex]->cardTagged = FALSE;

                    SetWindowPos(cardPtr[childIndex]->HWindow, HWND_TOP,
                        DISCARD_X, DISCARD_Y, CARDWIDTH, CARDHEIGHT,
                        SWP_NOSIZE);

                    InvalidateRect(cardPtr[childIndex]->HWindow,
                        NULL, TRUE);

                    discardCount++;

                    cardPtr[tagIdx]->where = INDISCARD;
                    cardPtr[tagIdx]->cardTagged = FALSE;

                    SetWindowPos(cardPtr[tagIdx]->HWindow, HWND_TOP,
                        DISCARD_X, DISCARD_Y, CARDWIDTH, CARDHEIGHT,
                        SWP_NOSIZE);

                    InvalidateRect(cardPtr[tagIdx]->HWindow,
                        NULL, TRUE);

                    discardCount++;

                    return TRUE;
                }
                else
                    {
                        MessageBeep(0);
                        isTagged = FALSE;
                        cardPtr[tagIdx]->cardTagged = FALSE;
                        InvalidateRect(cardPtr[tagIdx]->HWindow,
                            NULL, TRUE);
                    }
            }
            else
                {
                    cardPtr[childIndex]->cardTagged = TRUE;
                    isTagged = TRUE;
                    tagIdx = childIndex;
                    InvalidateRect(cardPtr[childIndex]->HWindow, NULL, TRUE);
                }
        }
    return FALSE;
}



//-------------------------------------------------------------------
//  CLASS:              TPyramidWin
//
//  MEMBER:             GetValue
//
//  DESCRIPTION:        Returns the actual card value
//
//  ARGUMENTS:          Value - bitmap index or deck index
//
//  RETURNS:            actual card value
//-------------------------------------------------------------------

int TPyramidWin::GetValue(int Value)
{
    if ( Value < 14 )
        return Value;
    else
        if ( Value < 27 )
            return (Value - 13);
    else
        if ( Value < 40 )
            return (Value - 26);
    else
        return (Value - 39);
}


//-------------------------------------------------------------------
//  CLASS:              TPyramidWin
//
//  MEMBER:             Animate
//
//  DESCRIPTION:        Move bitmap across the screen when user wins
//
//  ARGUMENTS:          NONE
//
//  RETURNS:            NONE
//-------------------------------------------------------------------

void TPyramidWin::Animate()
{
    PTWindowsObject animateWin = new TCardWin (this, NULL, 0, 390, 12,
        GetApplication()->hInstance);

    GetApplication()->MakeWindow(animateWin);

    HBITMAP winBitmap = LoadBitmap(GetApplication()->hInstance,
        MAKEINTRESOURCE(WINBITMSG));

    HDC hDC = GetDC(animateWin->HWindow);
    HDC hMemDC = CreateCompatibleDC(hDC);
    HDC TempDC = SelectObject(hMemDC, winBitmap);

    SetCapture(HWindow);

    //  Resize

    SetWindowPos(animateWin->HWindow, HWND_TOP, 390, 12, 174, 260,
        SWP_NOMOVE);

    for (int bx = 390; bx > 40; bx--)
    {
        MoveWindow(animateWin->HWindow, bx, 12, 174, 260, FALSE);
        BitBlt(hDC, 0, 0, 174, 260, hMemDC, 0, 0, SRCCOPY);
    }

    ReleaseCapture();

    SelectObject(hMemDC, TempDC);
    DeleteDC(hMemDC);
    DeleteObject(winBitmap);
    ReleaseDC(HWindow, hDC);
    delete(animateWin);
}


//-------------------------------------------------------------------
//  CLASS:              TPyramidWin
//
//  MEMBER:             WMNotifyClick
//
//  DESCRIPTION:        User defined message received when user
//                      clicks on a card
//
//  ARGUMENTS:          Windows user defined message
//
//  RETURNS:            NONE
//-------------------------------------------------------------------

void TPyramidWin::WMNotifyClick(RTMessage Msg)
{
    int depend;
    childIndex = (int) Msg.WParam;

    BOOL bUpdated = FALSE;

    if ( !hasDealt )
    {
        MessageBeep(0);
        strcpy(statusMsg, "The cards must be dealt first");
        StatusText->SetText(statusMsg);
        return;
    }

    switch ( cardPtr[childIndex]->where )
    {
        //---------------------------------
        //  Card has been discarded
        //---------------------------------

        case INDISCARD:

            MessageBeep(0);
            break;

        //---------------------------------
        //  Card is in the deck pile
        //---------------------------------

        case INDECK:

            if ( isTagged )
            {
                isTagged = FALSE;
                cardPtr[tagIdx]->cardTagged = FALSE;
                InvalidateRect(cardPtr[tagIdx]->HWindow, NULL, TRUE);
            }

            if ( playOccupied )
            {
                playOccupied = FALSE;

                cardPtr[playIdx]->where  = INWASTE;
                cardPtr[playIdx]->faceUp = TRUE;

                SetWindowPos(cardPtr[playIdx]->HWindow, HWND_TOP,
                    WASTE_X, WASTE_Y, CARDWIDTH, CARDHEIGHT, SWP_NOSIZE); 

                InvalidateRect(cardPtr[childIndex]->HWindow, NULL, TRUE);
            }

            cardPtr[childIndex]->where  = INPLAY;
            cardPtr[childIndex]->faceUp = TRUE;

            SetWindowPos(cardPtr[childIndex]->HWindow, HWND_TOP,
                PLAY_X, PLAY_Y, CARDWIDTH, CARDHEIGHT, SWP_NOSIZE); 

            InvalidateRect(cardPtr[childIndex]->HWindow, NULL, TRUE);

            playIdx = childIndex;
            playOccupied = TRUE;
            break;                         

        //---------------------------------
        //  Card is in the Pyramid
        //---------------------------------

        case INPYRAMID:

            if ( cardPtr[childIndex]->faceUp )
            {
                if ( !isTagged )
                {
                    if ( !(bUpdated = TestKing()) )
                        bUpdated = ProcessTag();
                }
                else
                    bUpdated = ProcessTag();
            }
            else
                MessageBeep(0);
            break;

        //---------------------------------
        //  Card is in the play pile
        //---------------------------------

        case INPLAY:

            if ( !isTagged )
            {
                if ( !(bUpdated = TestKing()) )
                    bUpdated = ProcessTag();
            }
            else
                bUpdated = ProcessTag();
            break;

        //---------------------------------
        //  Card is in the waste pile
        //---------------------------------

        case INWASTE:

            if ( (!playOccupied) ||
               ( (isTagged)      &&
                 (cardPtr[tagIdx]->where == INPLAY) ) )
            {
                bUpdated = ProcessTag();
            }
            else if ( (!playOccupied) && (!isTagged) )
                bUpdated = TestKing();
            else
                MessageBeep(0);
            break;
    }

    //----------------------------
    //  Open card if dependents
    //  have been discarded
    //----------------------------

    if ( bUpdated )
    {
        isTagged = FALSE;

        for ( int loop = 1; loop < 29; loop++)
        {
            if ( cardPtr[loop]->where == INPYRAMID )
            {
                if ( !cardPtr[loop]->faceUp )
                {
                    depend = cardPtr[loop]->dependentCard;

                    if ( (cardPtr[depend]->where == INDISCARD) &&
                        (cardPtr[depend + 1]->where == INDISCARD) )
                    {
                        cardPtr[loop]->faceUp = TRUE;
                        InvalidateRect(cardPtr[loop]->HWindow, NULL, TRUE);
                    }
                }
            }
        }
    }
                    

    //----------------------------
    //  Did the user win yet
    //----------------------------

    if ( discardCount >= 52 )
    {
        Animate();
        PlayAgain();
    }
} 


/*******************************************************************
 *******************************************************************
 *************************  MENU EVENTS  ***************************
 *******************************************************************
 *******************************************************************/
//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           CMQuit
//
//  DESCRIPTION:      Quit the application
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::CMQuit(RTMessage)
{
    CloseWindow();
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           CMAbout
//
//  DESCRIPTION:      Bring up the "about dialog box" using
//                    the class CDialog in order to center
//                    the dialog in the parent window
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::CMAbout(RTMessage)
{
    GetApplication()->ExecDialog(new CDialog(this, DIALOGABOUT));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           CMInstruct
//
//  DESCRIPTION:      Instructions on playing Pyramid
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::CMInstruct(RTMessage)
{
    GetApplication()->ExecDialog(new CDialog(this, DIALOGINSTRUCT));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           CMDeal
//
//  DESCRIPTION:      Deal the deck of cards
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::CMDeal(RTMessage)
{
    DealCards();
}
     

/*******************************************************************
 *******************************************************************
 **********************  PUSH BUTTON EVENTS  ***********************
 *******************************************************************
 *******************************************************************/
//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           IDQuit
//
//  DESCRIPTION:      Quit the application via the button
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::IDQuit(RTMessage)
{
    CloseWindow();
}

//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           IDInstruct
//
//  DESCRIPTION:      Bring up help via button
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::IDInstruct(RTMessage)
{
    GetApplication()->ExecDialog(new CDialog(this, DIALOGINSTRUCT));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           IDAbout
//
//  DESCRIPTION:      About Pyramid via button
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::IDAbout(RTMessage)
{
    GetApplication()->ExecDialog(new CDialog(this, DIALOGABOUT));
}


//-------------------------------------------------------------------
//  CLASS:            TPyramidWin
//
//  MEMBER:           IDDeal
//
//  DESCRIPTION:      Deal cards via button
//
//  ARGUMENTS:        Windows message
//
//  RETURNS:          NONE
//-------------------------------------------------------------------

void TPyramidWin::IDDeal(RTMessage)
{
    DealCards();
}


/*******************************************************************
 *******************************************************************
 ********************  MAIN CALLING ROUTINE  ***********************
 *******************************************************************
 *******************************************************************/

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{
    //----------------------------
    //  Load DLLs
    //----------------------------

    HINSTANCE hLib1 = LoadLibrary("BWCC.DLL");

    TPyramidApp PyramidApp("PyramidApp", hInstance, hPrevInstance,
        lpCmdLine, nCmdShow);

    PyramidApp.Run();

    //----------------------------
    //  Unload DLLs
    //----------------------------

    FreeLibrary(hLib1);

    return PyramidApp.Status;
}